package ru.rutoken.Samples.rtJNA;

import com.sun.jna.Memory;
import com.sun.jna.Native;
import com.sun.jna.ptr.IntByReference;
import ru.rutoken.Pkcs11.*;


class RtPkcs11Ex {
    public static Pkcs11Ex INSTANCE = (Pkcs11Ex) Native.loadLibrary("rtPKCS11ECP", Pkcs11Ex.class);
    RtPkcs11Ex() {}
    public static Pkcs11Ex getInstance() {
        return INSTANCE;
    }
}

public class ExtendedFunctions {

    private final static byte[] USER_PIN = {'1', '2', '3', '4', '5', '6', '7', '8'};
    private final static byte[] NEW_PIN = {'1', '2', '3', '4', '5', '6', '7', '8', '9'};
    private final static byte[] SO_PIN = {'8', '7', '6', '5', '4', '3', '2', '1'};
    private final static String TOKEN_LABEL = "It is my Rutoken label sample ";
    private final static String TOKEN_LONG_LABEL = "!!!Sample of Rutoken long-long-long-long-long label!!!";
    private final static int MAX_ADMIN_RETRY_COUNT = 10;
    private final static int MAX_USER_RETRY_COUNT = 10;

    public static void main(String[] args) {
        int hSession =-1;
        try {
            System.out.println("Example of rtPKCS11ECP.dll extended functions usage via JNA");

            System.out.println("Library initialization and getting of function list");
            int rv = RtPkcs11Library.getInstance().C_Initialize(null);
            if (rv != 0)
            {
                throw new Exception("Failed to initialize library, error code:" + Integer.toHexString(rv));
            }

            System.out.println("Getting list of slots with connected tokens");

            IntByReference slotsCount = new IntByReference();
            slotsCount.setValue(5);
            int[] pSlotList = new int[5];

            rv = RtPkcs11Library.getInstance().C_GetSlotList(true, null, slotsCount);
            if (rv != 0)
            {
                throw new Exception("C_GetSlotList failed, error code: " + Integer.toHexString(rv));
            }
            if (slotsCount.getValue() == 0)
            {
                throw new Exception("No Rutoken ECP is available!");
            }

            rv = RtPkcs11Library.getInstance().C_GetSlotList(true, pSlotList, slotsCount);
            if (rv != 0)
            {
                throw new Exception("C_GetSlotList failed, error code: " + Integer.toHexString(rv));
            }

            System.out.println("Getting of token extended info");
            CK_TOKEN_INFO_EXTENDED tokenInfoEx = new CK_TOKEN_INFO_EXTENDED();
            tokenInfoEx.ulSizeofThisStructure = tokenInfoEx.size();
            rv = RtPkcs11Ex.getInstance().C_EX_GetTokenInfoExtended(pSlotList[0], tokenInfoEx);
            if (rv != 0)
            {
                throw new Exception("C_EX_GetTokenInfoExtended failed, error code: " + Integer.toHexString(rv));
            }

            System.out.println("Token info:");
            System.out.printf(" Token type: \t\t\t\t0x%08X ", tokenInfoEx.ulTokenType);
            if (tokenInfoEx.ulTokenType == RtPkcs11Constants.TOKEN_TYPE_RUTOKEN_ECP)
                System.out.println("(Rutoken ECP)");
            if (tokenInfoEx.ulTokenType == RtPkcs11Constants.TOKEN_TYPE_RUTOKEN_LITE)
                System.out.println("(Rutoken LITE)");
            if (tokenInfoEx.ulTokenType == RtPkcs11Constants.TOKEN_TYPE_RUTOKEN)
                System.out.println("(Rutoken)");
            System.out.printf(" Protocol number: \t\t\t0x%08X \n", tokenInfoEx.ulProtocolNumber);
            System.out.printf(" Microcode number:  \t\t0x%08X \n", tokenInfoEx.ulMicrocodeNumber);
            System.out.printf(" Flags: \t\t\t\t\t0x%08X\n", tokenInfoEx.flags);
            System.out.printf(" Min Admin PIN length: \t\t0x%08X \n", tokenInfoEx.ulMinAdminPinLen);
            System.out.printf(" Max User PIN length: \t\t0x%08X\n", tokenInfoEx.ulMaxUserPinLen);
            System.out.printf(" Min User PIN length: \t\t0x%08X\n", tokenInfoEx.ulMinUserPinLen);
            System.out.printf(" Max Admin login failures: \t0x%08X \n", tokenInfoEx.ulMaxAdminRetryCount);
            System.out.printf(" Admin login failures: \t\t0x%08X \n", tokenInfoEx.ulAdminRetryCountLeft);
            System.out.printf(" Max User login failures: \t0x%08X \n", tokenInfoEx.ulMaxUserRetryCount);
            System.out.printf(" User login failures: \t\t0x%08X \n", tokenInfoEx.ulUserRetryCountLeft);
            System.out.printf(" Token SN: \t\t\t\t\t0x");
            for (int i = 4; i < tokenInfoEx.serialNumber.length; i++)
                System.out.printf("%02X", tokenInfoEx.serialNumber[i]);
            System.out.println();
            System.out.printf(" Total memory: \t\t\t\t0x%08X\n", tokenInfoEx.ulTotalMemory);
            System.out.printf(" Free  memory: \t\t\t\t0x%08X\n", tokenInfoEx.ulFreeMemory);
            System.out.printf(" ATR: \t\t\t\t\t\t");
            for (int i = 0; i < tokenInfoEx.ulATRLen; i++)
                System.out.printf("%02X", tokenInfoEx.ATR[i]);
            System.out.printf(" (");
            for (int i = 3; i < tokenInfoEx.ulATRLen-1; i++)
                System.out.printf("%c", (char)tokenInfoEx.ATR[i] );
            System.out.println(")");

            System.out.println("Token initialization");
            CK_RUTOKEN_INIT_PARAM initInfo = new CK_RUTOKEN_INIT_PARAM();
            initInfo.UseRepairMode = 0;
            initInfo.pNewAdminPin = new Memory(SO_PIN.length);
            initInfo.pNewAdminPin.write(0, SO_PIN, 0, SO_PIN.length);
            initInfo.ulNewAdminPinLen = SO_PIN.length;
            initInfo.pNewUserPin = new Memory(NEW_PIN.length);
            initInfo.pNewUserPin.write(0, NEW_PIN, 0, NEW_PIN.length);
            initInfo.ulNewUserPinLen = NEW_PIN.length;
            initInfo.ulMinAdminPinLen = 6;
            initInfo.ulMinUserPinLen = 6;
            initInfo.ChangeUserPINPolicy = RtPkcs11Constants.TOKEN_FLAGS_ADMIN_CHANGE_USER_PIN |
                                           RtPkcs11Constants.TOKEN_FLAGS_USER_CHANGE_USER_PIN;
            initInfo.ulMaxAdminRetryCount = MAX_ADMIN_RETRY_COUNT;
            initInfo.ulMaxUserRetryCount = MAX_USER_RETRY_COUNT;
            initInfo.pTokenLabel = new Memory(TOKEN_LABEL.length());
            initInfo.pTokenLabel.write(0, TOKEN_LABEL.getBytes(), 0, TOKEN_LABEL.length());
            initInfo.ulLabelLen = TOKEN_LABEL.length();
            initInfo.ulSizeofThisStructure = initInfo.size();

            rv = RtPkcs11Ex.getInstance().C_EX_InitToken(pSlotList[0], SO_PIN, SO_PIN.length, initInfo);
            if (rv != 0)
            {
                throw new Exception("C_EX_InitToken failed, error code: " + Integer.toHexString(rv));
            }

            System.out.println("Opening of session");
            IntByReference phSession = new IntByReference();
            rv = RtPkcs11Library.getInstance().C_OpenSession(pSlotList[0],
                    Pkcs11Constants.CKF_SERIAL_SESSION | Pkcs11Constants.CKF_RW_SESSION, null, null, phSession);
            if (rv != 0)
            {
                throw new Exception("C_OpenSession failed, error code: " + Integer.toHexString(rv));
            }
            hSession = phSession.getValue();

            System.out.println("Logging in as administrator");
            rv = RtPkcs11Library.getInstance().C_Login(hSession, Pkcs11Constants.CKU_SO, SO_PIN, SO_PIN.length);
            if (rv != 0)
            {
                throw new Exception("C_Login failed, error code: " + Integer.toHexString(rv));
            }

            System.out.println("User PIN unlocking");
            rv = RtPkcs11Ex.getInstance().C_EX_UnblockUserPIN(hSession);
            if (rv != 0)
            {
                throw new Exception("C_EX_UnblockUserPIN failed, error code: " + Integer.toHexString(rv));
            }

            System.out.println("Logging out");
            rv = RtPkcs11Library.getInstance().C_Logout(hSession);
            if (rv != 0)
            {
                throw new Exception("C_Logout failed, error code: " + Integer.toHexString(rv));
            }

            System.out.println("Logging in as user");
            rv = RtPkcs11Library.getInstance().C_Login(hSession, Pkcs11Constants.CKU_USER, NEW_PIN, NEW_PIN.length);
            if (rv != 0)
            {
                throw new Exception("C_Login failed, error code: " + Integer.toHexString(rv));
            }

            System.out.println("Setting of token name");
            rv = RtPkcs11Ex.getInstance().C_EX_SetTokenName(hSession, TOKEN_LONG_LABEL.getBytes(),
                    TOKEN_LONG_LABEL.length());
            if (rv != 0)
            {
                throw new Exception("C_EX_SetTokenName failed, error code: " + Integer.toHexString(rv));
            }

            System.out.println("User PIN changing");
            rv = RtPkcs11Library.getInstance().C_SetPIN(hSession, NEW_PIN, NEW_PIN.length, USER_PIN, USER_PIN.length);
            if (rv != 0)
            {
                throw new Exception("C_SetPIN failed, error code: " + Integer.toHexString(rv));
            }

            System.out.println("Logging out");
            rv = RtPkcs11Library.getInstance().C_Logout(hSession);
            if (rv != 0)
            {
                throw new Exception("C_Logout failed, error code: " + Integer.toHexString(rv));
            }

            System.out.println("Closing of session");
            rv = RtPkcs11Library.getInstance().C_CloseSession(hSession);
            if (rv != 0)
            {
                throw new Exception("C_CloseSession failed, error code: " + Integer.toHexString(rv));
            }
            hSession = -1;

            System.out.println("Test has been completed successfully.\n");
        }
        catch(Exception e){
            System.out.printf(e.getMessage() + "\nSome error occurred.");
        }
        try {
            RtPkcs11Library.getInstance().C_Logout(hSession);
        }
        catch(Exception e){}
        try {
            RtPkcs11Library.getInstance().C_CloseSession(hSession);
        }
        catch(Exception e){}
        try {
            RtPkcs11Library.getInstance().C_Finalize(null);
        }
        catch(Exception e){}
    }
}

