package ru.rutoken.Samples.rtJNA;

import com.sun.jna.Memory;
import com.sun.jna.Pointer;
import com.sun.jna.ptr.IntByReference;
import ru.rutoken.Pkcs11.*;
import sun.security.pkcs11.wrapper.*;
import sun.security.pkcs11.wrapper.CK_ATTRIBUTE;

public class PKIExtensions {

    static PKCS11 pkcs11Wrapper;

    // Значения ключевой пары ГОСТ Р 34.10-2001
    private final static byte[] cbPubKeyValue = {
        (byte)0xfb, (byte)0x3c, (byte)0xdc, (byte)0x59, (byte)0xc3, (byte)0x9c, (byte)0x4a, (byte)0x43,
        (byte)0x89, (byte)0x87, (byte)0xc7, (byte)0xd7, (byte)0xfe, (byte)0x50, (byte)0x19, (byte)0xb3,
        (byte)0x0c, (byte)0x8b, (byte)0x76, (byte)0x97, (byte)0xa9, (byte)0xdf, (byte)0xb7, (byte)0xca,
        (byte)0x2c, (byte)0x6c, (byte)0x3b, (byte)0xa9, (byte)0x13, (byte)0xf4, (byte)0xe0, (byte)0x69,
        (byte)0x02, (byte)0x59, (byte)0x92, (byte)0x47, (byte)0x21, (byte)0x1a, (byte)0xef, (byte)0x90,
        (byte)0x61, (byte)0x91, (byte)0x40, (byte)0x30, (byte)0xdd, (byte)0x7c, (byte)0xb0, (byte)0x4f,
        (byte)0x64, (byte)0x5f, (byte)0x24, (byte)0x9a, (byte)0xf1, (byte)0xd6, (byte)0x0f, (byte)0xa9,
        (byte)0xf0, (byte)0x86, (byte)0xd9, (byte)0x35, (byte)0x2b, (byte)0x3e, (byte)0xf2, (byte)0xf3,
    };

    private final static byte[] cbPrvKeyValue = {
        (byte)0x8b, (byte)0x1d, (byte)0x96, (byte)0xf7, (byte)0x53, (byte)0x7e, (byte)0x69, (byte)0xe2,
        (byte)0xa6, (byte)0x5e, (byte)0xf1, (byte)0xf4, (byte)0xbd, (byte)0xed, (byte)0x84, (byte)0xb4,
        (byte)0xa9, (byte)0x60, (byte)0xba, (byte)0xdd, (byte)0x7b, (byte)0x9f, (byte)0x27, (byte)0x5f,
        (byte)0x68, (byte)0xe3, (byte)0x5e, (byte)0x1a, (byte)0x5c, (byte)0x6e, (byte)0xa4, (byte)0xf6,
    };

    // Значение сертификата ключа подписи
    private final static byte[] cbCertificate = {
        (byte)0x30, (byte)0x82, (byte)0x03, (byte)0x36, (byte)0x30, (byte)0x82, (byte)0x02, (byte)0xe5,
        (byte)0xa0, (byte)0x03, (byte)0x02, (byte)0x01, (byte)0x02, (byte)0x02, (byte)0x0a, (byte)0x24,
        (byte)0xa5, (byte)0x24, (byte)0x63, (byte)0x00, (byte)0x02, (byte)0x00, (byte)0x01, (byte)0xa7,
        (byte)0x15, (byte)0x30, (byte)0x08, (byte)0x06, (byte)0x06, (byte)0x2a, (byte)0x85, (byte)0x03,
        (byte)0x02, (byte)0x02, (byte)0x03, (byte)0x30, (byte)0x65, (byte)0x31, (byte)0x20, (byte)0x30,
        (byte)0x1e, (byte)0x06, (byte)0x09, (byte)0x2a, (byte)0x86, (byte)0x48, (byte)0x86, (byte)0xf7,
        (byte)0x0d, (byte)0x01, (byte)0x09, (byte)0x01, (byte)0x16, (byte)0x11, (byte)0x69, (byte)0x6e,
        (byte)0x66, (byte)0x6f, (byte)0x40, (byte)0x63, (byte)0x72, (byte)0x79, (byte)0x70, (byte)0x74,
        (byte)0x6f, (byte)0x70, (byte)0x72, (byte)0x6f, (byte)0x2e, (byte)0x72, (byte)0x75, (byte)0x31,
        (byte)0x0b, (byte)0x30, (byte)0x09, (byte)0x06, (byte)0x03, (byte)0x55, (byte)0x04, (byte)0x06,
        (byte)0x13, (byte)0x02, (byte)0x52, (byte)0x55, (byte)0x31, (byte)0x13, (byte)0x30, (byte)0x11,
        (byte)0x06, (byte)0x03, (byte)0x55, (byte)0x04, (byte)0x0a, (byte)0x13, (byte)0x0a, (byte)0x43,
        (byte)0x52, (byte)0x59, (byte)0x50, (byte)0x54, (byte)0x4f, (byte)0x2d, (byte)0x50, (byte)0x52,
        (byte)0x4f, (byte)0x31, (byte)0x1f, (byte)0x30, (byte)0x1d, (byte)0x06, (byte)0x03, (byte)0x55,
        (byte)0x04, (byte)0x03, (byte)0x13, (byte)0x16, (byte)0x54, (byte)0x65, (byte)0x73, (byte)0x74,
        (byte)0x20, (byte)0x43, (byte)0x65, (byte)0x6e, (byte)0x74, (byte)0x65, (byte)0x72, (byte)0x20,
        (byte)0x43, (byte)0x52, (byte)0x59, (byte)0x50, (byte)0x54, (byte)0x4f, (byte)0x2d, (byte)0x50,
        (byte)0x52, (byte)0x4f, (byte)0x30, (byte)0x1e, (byte)0x17, (byte)0x0d, (byte)0x31, (byte)0x31,
        (byte)0x31, (byte)0x31, (byte)0x32, (byte)0x32, (byte)0x31, (byte)0x30, (byte)0x31, (byte)0x33,
        (byte)0x34, (byte)0x32, (byte)0x5a, (byte)0x17, (byte)0x0d, (byte)0x31, (byte)0x34, (byte)0x31,
        (byte)0x30, (byte)0x30, (byte)0x34, (byte)0x30, (byte)0x37, (byte)0x30, (byte)0x39, (byte)0x34,
        (byte)0x31, (byte)0x5a, (byte)0x30, (byte)0x65, (byte)0x31, (byte)0x10, (byte)0x30, (byte)0x0e,
        (byte)0x06, (byte)0x03, (byte)0x55, (byte)0x04, (byte)0x03, (byte)0x13, (byte)0x07, (byte)0x49,
        (byte)0x76, (byte)0x61, (byte)0x6e, (byte)0x6f, (byte)0x66, (byte)0x66, (byte)0x31, (byte)0x0b,
        (byte)0x30, (byte)0x09, (byte)0x06, (byte)0x03, (byte)0x55, (byte)0x04, (byte)0x06, (byte)0x13,
        (byte)0x02, (byte)0x52, (byte)0x55, (byte)0x31, (byte)0x14, (byte)0x30, (byte)0x12, (byte)0x06,
        (byte)0x03, (byte)0x55, (byte)0x04, (byte)0x05, (byte)0x13, (byte)0x0b, (byte)0x31, (byte)0x32,
        (byte)0x33, (byte)0x31, (byte)0x32, (byte)0x33, (byte)0x31, (byte)0x32, (byte)0x33, (byte)0x31,
        (byte)0x32, (byte)0x31, (byte)0x1d, (byte)0x30, (byte)0x1b, (byte)0x06, (byte)0x09, (byte)0x2a,
        (byte)0x86, (byte)0x48, (byte)0x86, (byte)0xf7, (byte)0x0d, (byte)0x01, (byte)0x09, (byte)0x01,
        (byte)0x16, (byte)0x0e, (byte)0x69, (byte)0x76, (byte)0x61, (byte)0x6e, (byte)0x6f, (byte)0x76,
        (byte)0x40, (byte)0x6d, (byte)0x61, (byte)0x69, (byte)0x6c, (byte)0x2e, (byte)0x72, (byte)0x75,
        (byte)0x31, (byte)0x0f, (byte)0x30, (byte)0x0d, (byte)0x06, (byte)0x03, (byte)0x55, (byte)0x04,
        (byte)0x08, (byte)0x13, (byte)0x06, (byte)0x4d, (byte)0x6f, (byte)0x73, (byte)0x63, (byte)0x6f,
        (byte)0x77, (byte)0x30, (byte)0x63, (byte)0x30, (byte)0x1c, (byte)0x06, (byte)0x06, (byte)0x2a,
        (byte)0x85, (byte)0x03, (byte)0x02, (byte)0x02, (byte)0x13, (byte)0x30, (byte)0x12, (byte)0x06,
        (byte)0x07, (byte)0x2a, (byte)0x85, (byte)0x03, (byte)0x02, (byte)0x02, (byte)0x23, (byte)0x01,
        (byte)0x06, (byte)0x07, (byte)0x2a, (byte)0x85, (byte)0x03, (byte)0x02, (byte)0x02, (byte)0x1e,
        (byte)0x01, (byte)0x03, (byte)0x43, (byte)0x00, (byte)0x04, (byte)0x40, (byte)0xfb, (byte)0x3c,
        (byte)0xdc, (byte)0x59, (byte)0xc3, (byte)0x9c, (byte)0x4a, (byte)0x43, (byte)0x89, (byte)0x87,
        (byte)0xc7, (byte)0xd7, (byte)0xfe, (byte)0x50, (byte)0x19, (byte)0xb3, (byte)0x0c, (byte)0x8b,
        (byte)0x76, (byte)0x97, (byte)0xa9, (byte)0xdf, (byte)0xb7, (byte)0xca, (byte)0x2c, (byte)0x6c,
        (byte)0x3b, (byte)0xa9, (byte)0x13, (byte)0xf4, (byte)0xe0, (byte)0x69, (byte)0x02, (byte)0x59,
        (byte)0x92, (byte)0x47, (byte)0x21, (byte)0x1a, (byte)0xef, (byte)0x90, (byte)0x61, (byte)0x91,
        (byte)0x40, (byte)0x30, (byte)0xdd, (byte)0x7c, (byte)0xb0, (byte)0x4f, (byte)0x64, (byte)0x5f,
        (byte)0x24, (byte)0x9a, (byte)0xf1, (byte)0xd6, (byte)0x0f, (byte)0xa9, (byte)0xf0, (byte)0x86,
        (byte)0xd9, (byte)0x35, (byte)0x2b, (byte)0x3e, (byte)0xf2, (byte)0xf3, (byte)0xa3, (byte)0x82,
        (byte)0x01, (byte)0x73, (byte)0x30, (byte)0x82, (byte)0x01, (byte)0x6f, (byte)0x30, (byte)0x0b,
        (byte)0x06, (byte)0x03, (byte)0x55, (byte)0x1d, (byte)0x0f, (byte)0x04, (byte)0x04, (byte)0x03,
        (byte)0x02, (byte)0x04, (byte)0xf0, (byte)0x30, (byte)0x26, (byte)0x06, (byte)0x03, (byte)0x55,
        (byte)0x1d, (byte)0x25, (byte)0x04, (byte)0x1f, (byte)0x30, (byte)0x1d, (byte)0x06, (byte)0x07,
        (byte)0x2a, (byte)0x85, (byte)0x03, (byte)0x02, (byte)0x02, (byte)0x22, (byte)0x06, (byte)0x06,
        (byte)0x08, (byte)0x2b, (byte)0x06, (byte)0x01, (byte)0x05, (byte)0x05, (byte)0x07, (byte)0x03,
        (byte)0x02, (byte)0x06, (byte)0x08, (byte)0x2b, (byte)0x06, (byte)0x01, (byte)0x05, (byte)0x05,
        (byte)0x07, (byte)0x03, (byte)0x04, (byte)0x30, (byte)0x1d, (byte)0x06, (byte)0x03, (byte)0x55,
        (byte)0x1d, (byte)0x0e, (byte)0x04, (byte)0x16, (byte)0x04, (byte)0x14, (byte)0x8a, (byte)0x24,
        (byte)0x35, (byte)0x74, (byte)0x6b, (byte)0xf7, (byte)0x91, (byte)0x17, (byte)0x92, (byte)0xb2,
        (byte)0xcf, (byte)0x8f, (byte)0x63, (byte)0x87, (byte)0xb7, (byte)0x69, (byte)0x06, (byte)0xe1,
        (byte)0x71, (byte)0xf2, (byte)0x30, (byte)0x1f, (byte)0x06, (byte)0x03, (byte)0x55, (byte)0x1d,
        (byte)0x23, (byte)0x04, (byte)0x18, (byte)0x30, (byte)0x16, (byte)0x80, (byte)0x14, (byte)0x6d,
        (byte)0x8f, (byte)0x5e, (byte)0x05, (byte)0xd9, (byte)0x5f, (byte)0xac, (byte)0x91, (byte)0x17,
        (byte)0x94, (byte)0x1e, (byte)0x95, (byte)0x9a, (byte)0x05, (byte)0x30, (byte)0x38, (byte)0x37,
        (byte)0x7a, (byte)0x10, (byte)0x2a, (byte)0x30, (byte)0x55, (byte)0x06, (byte)0x03, (byte)0x55,
        (byte)0x1d, (byte)0x1f, (byte)0x04, (byte)0x4e, (byte)0x30, (byte)0x4c, (byte)0x30, (byte)0x4a,
        (byte)0xa0, (byte)0x48, (byte)0xa0, (byte)0x46, (byte)0x86, (byte)0x44, (byte)0x68, (byte)0x74,
        (byte)0x74, (byte)0x70, (byte)0x3a, (byte)0x2f, (byte)0x2f, (byte)0x77, (byte)0x77, (byte)0x77,
        (byte)0x2e, (byte)0x63, (byte)0x72, (byte)0x79, (byte)0x70, (byte)0x74, (byte)0x6f, (byte)0x70,
        (byte)0x72, (byte)0x6f, (byte)0x2e, (byte)0x72, (byte)0x75, (byte)0x2f, (byte)0x43, (byte)0x65,
        (byte)0x72, (byte)0x74, (byte)0x45, (byte)0x6e, (byte)0x72, (byte)0x6f, (byte)0x6c, (byte)0x6c,
        (byte)0x2f, (byte)0x54, (byte)0x65, (byte)0x73, (byte)0x74, (byte)0x25, (byte)0x32, (byte)0x30,
        (byte)0x43, (byte)0x65, (byte)0x6e, (byte)0x74, (byte)0x65, (byte)0x72, (byte)0x25, (byte)0x32,
        (byte)0x30, (byte)0x43, (byte)0x52, (byte)0x59, (byte)0x50, (byte)0x54, (byte)0x4f, (byte)0x2d,
        (byte)0x50, (byte)0x52, (byte)0x4f, (byte)0x28, (byte)0x32, (byte)0x29, (byte)0x2e, (byte)0x63,
        (byte)0x72, (byte)0x6c, (byte)0x30, (byte)0x81, (byte)0xa0, (byte)0x06, (byte)0x08, (byte)0x2b,
        (byte)0x06, (byte)0x01, (byte)0x05, (byte)0x05, (byte)0x07, (byte)0x01, (byte)0x01, (byte)0x04,
        (byte)0x81, (byte)0x93, (byte)0x30, (byte)0x81, (byte)0x90, (byte)0x30, (byte)0x33, (byte)0x06,
        (byte)0x08, (byte)0x2b, (byte)0x06, (byte)0x01, (byte)0x05, (byte)0x05, (byte)0x07, (byte)0x30,
        (byte)0x01, (byte)0x86, (byte)0x27, (byte)0x68, (byte)0x74, (byte)0x74, (byte)0x70, (byte)0x3a,
        (byte)0x2f, (byte)0x2f, (byte)0x77, (byte)0x77, (byte)0x77, (byte)0x2e, (byte)0x63, (byte)0x72,
        (byte)0x79, (byte)0x70, (byte)0x74, (byte)0x6f, (byte)0x70, (byte)0x72, (byte)0x6f, (byte)0x2e,
        (byte)0x72, (byte)0x75, (byte)0x2f, (byte)0x6f, (byte)0x63, (byte)0x73, (byte)0x70, (byte)0x6e,
        (byte)0x63, (byte)0x2f, (byte)0x6f, (byte)0x63, (byte)0x73, (byte)0x70, (byte)0x2e, (byte)0x73,
        (byte)0x72, (byte)0x66, (byte)0x30, (byte)0x59, (byte)0x06, (byte)0x08, (byte)0x2b, (byte)0x06,
        (byte)0x01, (byte)0x05, (byte)0x05, (byte)0x07, (byte)0x30, (byte)0x02, (byte)0x86, (byte)0x4d,
        (byte)0x68, (byte)0x74, (byte)0x74, (byte)0x70, (byte)0x3a, (byte)0x2f, (byte)0x2f, (byte)0x77,
        (byte)0x77, (byte)0x77, (byte)0x2e, (byte)0x63, (byte)0x72, (byte)0x79, (byte)0x70, (byte)0x74,
        (byte)0x6f, (byte)0x70, (byte)0x72, (byte)0x6f, (byte)0x2e, (byte)0x72, (byte)0x75, (byte)0x2f,
        (byte)0x43, (byte)0x65, (byte)0x72, (byte)0x74, (byte)0x45, (byte)0x6e, (byte)0x72, (byte)0x6f,
        (byte)0x6c, (byte)0x6c, (byte)0x2f, (byte)0x70, (byte)0x6b, (byte)0x69, (byte)0x2d, (byte)0x73,
        (byte)0x69, (byte)0x74, (byte)0x65, (byte)0x5f, (byte)0x54, (byte)0x65, (byte)0x73, (byte)0x74,
        (byte)0x25, (byte)0x32, (byte)0x30, (byte)0x43, (byte)0x65, (byte)0x6e, (byte)0x74, (byte)0x65,
        (byte)0x72, (byte)0x25, (byte)0x32, (byte)0x30, (byte)0x43, (byte)0x52, (byte)0x59, (byte)0x50,
        (byte)0x54, (byte)0x4f, (byte)0x2d, (byte)0x50, (byte)0x52, (byte)0x4f, (byte)0x28, (byte)0x32,
        (byte)0x29, (byte)0x2e, (byte)0x63, (byte)0x72, (byte)0x74, (byte)0x30, (byte)0x08, (byte)0x06,
        (byte)0x06, (byte)0x2a, (byte)0x85, (byte)0x03, (byte)0x02, (byte)0x02, (byte)0x03, (byte)0x03,
        (byte)0x41, (byte)0x00, (byte)0x2b, (byte)0xd2, (byte)0xfe, (byte)0x64, (byte)0x54, (byte)0x3a,
        (byte)0xe1, (byte)0xf6, (byte)0x89, (byte)0x75, (byte)0xfe, (byte)0xbb, (byte)0xa6, (byte)0x29,
        (byte)0xed, (byte)0x0b, (byte)0x92, (byte)0xc0, (byte)0xa4, (byte)0x84, (byte)0x15, (byte)0x59,
        (byte)0x23, (byte)0x12, (byte)0x08, (byte)0xbb, (byte)0xd3, (byte)0xab, (byte)0x8e, (byte)0x2e,
        (byte)0x75, (byte)0xb9, (byte)0xbf, (byte)0x9e, (byte)0xd1, (byte)0x9d, (byte)0x1e, (byte)0xf9,
        (byte)0x6a, (byte)0x24, (byte)0xed, (byte)0xb8, (byte)0x58, (byte)0x15, (byte)0x1f, (byte)0x03,
        (byte)0x11, (byte)0xfa, (byte)0xd3, (byte)0x85, (byte)0xf1, (byte)0x34, (byte)0x96, (byte)0xac,
        (byte)0x20, (byte)0x8e, (byte)0xdd, (byte)0xad, (byte)0x4e, (byte)0xae, (byte)0x55, (byte)0x3e,
        (byte)0x8d, (byte)0xd1, (byte)0xff,
    };

    private final static byte[] ATTR_CRYPTO_PRO_A = {
        0x06, 0x07, 0x2A, (byte)0x85, 0x03, 0x02, 0x02, 0x23, 0x01};

    // Шаблон для импорта открытого ключа ГОСТ Р 34.10-2001
    private final static CK_ATTRIBUTE[] PublicKeyTmpl = {
            new CK_ATTRIBUTE(PKCS11Constants.CKA_CLASS, PKCS11Constants.CKO_PUBLIC_KEY),
            new CK_ATTRIBUTE(PKCS11Constants.CKA_LABEL,
                    new String("Sample GOST R 34.10-2001 Public Key (Aktiv Co.)").toCharArray()),
            new CK_ATTRIBUTE(PKCS11Constants.CKA_ID,
                    new String("Sample GOST R 34.10-2001 keypair (Aktiv Co.)").toCharArray()),
            new CK_ATTRIBUTE(PKCS11Constants.CKA_KEY_TYPE, RtPkcs11Constants.CKK_GOSTR3410),
            new CK_ATTRIBUTE(PKCS11Constants.CKA_TOKEN, true),
            new CK_ATTRIBUTE(PKCS11Constants.CKA_PRIVATE, false),
            new CK_ATTRIBUTE(RtPkcs11Constants.CKA_GOSTR3410_PARAMS, ATTR_CRYPTO_PRO_A),
            new CK_ATTRIBUTE(PKCS11Constants.CKA_VALUE, cbPubKeyValue)
    };

    // Шаблон для импорта закрытого ключа ГОСТ Р 34.10-2001
    private final static CK_ATTRIBUTE[] PrivateKeyTmpl = {
        new CK_ATTRIBUTE(PKCS11Constants.CKA_CLASS, PKCS11Constants.CKO_PRIVATE_KEY),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_LABEL,
                new String("Sample GOST R 34.10-2001 Private Key (Aktiv Co.)").toCharArray()),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_ID,
                new String("Sample GOST R 34.10-2001 keypair (Aktiv Co.)").toCharArray()),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_KEY_TYPE, RtPkcs11Constants.CKK_GOSTR3410),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_TOKEN, true),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_PRIVATE, true),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_DERIVE, true),
        new CK_ATTRIBUTE(RtPkcs11Constants.CKA_GOSTR3410_PARAMS, ATTR_CRYPTO_PRO_A),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_VALUE, cbPrvKeyValue),
    };

    // Шаблон для импорта сертификата ключа подписи
    private final static CK_ATTRIBUTE[] CertTmpl = {
        new CK_ATTRIBUTE(PKCS11Constants.CKA_CLASS, PKCS11Constants.CKO_CERTIFICATE),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_ID,
                new String("Sample GOST R 34.10-2001 keypair (Aktiv Co.)").toCharArray()),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_TOKEN, true),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_PRIVATE, false),
        new CK_ATTRIBUTE(PKCS11Constants.CKA_VALUE, cbCertificate)
    };

    // Список полей DN (Distinguished Name)
    private final static String[] dn = 	{
            new String ("CN"),
            new String ("Ivanoff"),
            new String ("C"),
            new String ("RU"),
            new String ("2.5.4.5"),
            new String ("12312312312"),
            new String ("1.2.840.113549.1.9.1"),
            new String ("ivanov@mail.ru"),
            new String ("ST"),
            new String ("Moscow"),
   };

    // Список дополнительных полей
    private final static String[] exts = {
            new String ("keyUsage"),
            new String ("digitalSignature,nonRepudiation,keyEncipherment,dataEncipherment"),
            new String ("extendedKeyUsage"),
            new String ("1.2.643.2.2.34.6,1.3.6.1.5.5.7.3.2,1.3.6.1.5.5.7.3.4"),
    };

    // Данные для подписи
    private final static byte[] pbtData = { '1' };

    private final static char[] USER_PIN = {'1', '2', '3', '4', '5', '6', '7', '8'};

    public static void main(String[] args) {
        long session = -1;

        try {
            System.out.println("Example of rtPKCS11ECP.dll PKI extensions usage via sun.security.pkcs11.wrapper");

            System.out.println("Library initialization and getting of function list");
            pkcs11Wrapper = PKCS11.getInstance("rtPKCS11ECP", "C_GetFunctionList", new CK_C_INITIALIZE_ARGS(), false);

            System.out.println("Getting list of slots with connected tokens");
            long[] slotList = pkcs11Wrapper.C_GetSlotList(true);
            if (slotList.length == 0) {
                throw new Exception("No Rutoken ECP is available!");
            }

            System.out.println("Opening of session");
            session = pkcs11Wrapper.C_OpenSession(
                    slotList[0], PKCS11Constants.CKF_SERIAL_SESSION | PKCS11Constants.CKF_RW_SESSION, null, null);

            System.out.println("Logging in as user");
            pkcs11Wrapper.C_Login(session, PKCS11Constants.CKU_USER, USER_PIN);

            System.out.println("Creating public key");
            long hPubKey = pkcs11Wrapper.C_CreateObject(session, PublicKeyTmpl);

            System.out.println("Creating private key");
            long hPrvKey = pkcs11Wrapper.C_CreateObject(session, PrivateKeyTmpl);

            // Cоздание запроса на сертификат
            System.out.println("Creating certificate request");
            IntByReference ulCsrSize = new IntByReference();
            Pointer ppbtCsr = new Memory(Pointer.SIZE);
            int rv = RtPkcs11Ex.getInstance().C_EX_CreateCSR( (int)session, (int)hPubKey, dn, dn.length,
                    ppbtCsr, ulCsrSize, (int)hPrvKey, 0, 0, exts, exts.length
            );
            if (rv != 0)
            {
                throw new Exception("C_EX_CreateCSR failed, error code" + Integer.toHexString(rv));
            }
            Pointer pbtCsr = ppbtCsr.getPointer(0);
            System.out.println("Certificate request is: ");
            for (int i = 0; i < ulCsrSize.getValue(); i++)
            {
                System.out.printf(" %02X", pbtCsr.getByteArray(0, ulCsrSize.getValue())[i]);
                if ((i + 1) % 8 == 0)
                    System.out.printf("\n");
            }

            System.out.println("\nCreating certificate");
            long hCert = pkcs11Wrapper.C_CreateObject(session, CertTmpl);

            System.out.println("Getting info about certificate");
            IntByReference ulInfoSize = new IntByReference();
            Pointer ppbtInfo = new Memory(Pointer.SIZE);
            rv = RtPkcs11Ex.getInstance().C_EX_GetCertificateInfoText(
                    (int)session, (int)hCert, ppbtInfo, ulInfoSize);
            if (rv != 0)
            {
                throw new Exception("Failed on getting info about certificate, error code " + Integer.toHexString(rv));
            }
            Pointer pbtInfo = ppbtInfo.getPointer(0);
            System.out.println("Certificate is: ");
            for (int i = 0; i < ulInfoSize.getValue(); i++)
            {
                System.out.printf(" %02X", pbtInfo.getByteArray(0, ulInfoSize.getValue())[i]);
                if ((i + 1) % 8 == 0)
                    System.out.printf("\n");
            }

            System.out.println("\nPKCS7 signing");
            IntByReference ulSignatureLen = new IntByReference();
            Pointer pptSignature = new Memory(Pointer.SIZE);
            rv = RtPkcs11Ex.getInstance().C_EX_PKCS7Sign(
                    (int)session, pbtData, pbtData.length, (int)hCert,
                    pptSignature, ulSignatureLen, (int)hPrvKey, 0, 0, 0
                    );
            if (rv != 0)
            {
                throw new Exception("C_EX_PKCS7Sign failed, error code " + Integer.toHexString(rv));
            }
            Pointer pbtSignature = pptSignature.getPointer(0);

            System.out.println("Signed data is:");
            for (int i = 0; i < ulSignatureLen.getValue(); i++)
            {
                System.out.printf(" %02X", pbtSignature.getByteArray(0, ulSignatureLen.getValue())[i]);
                if ((i + 1) % 8 == 0)
                    System.out.printf("\n");
            }
            System.out.println("\nData has been signed successfully.");

            rv = RtPkcs11Ex.getInstance().C_EX_FreeBuffer(ppbtCsr.getPointer(0));
            if (rv != 0)
            {
                throw new Exception("C_EX_FreeBuffer failed, error code" + Integer.toHexString(rv));
            }

            rv = RtPkcs11Ex.getInstance().C_EX_FreeBuffer(pptSignature.getPointer(0));
            if (rv != 0)
            {
                throw new Exception("C_EX_FreeBuffer failed, error code" + Integer.toHexString(rv));
            }

            rv = RtPkcs11Ex.getInstance().C_EX_FreeBuffer(ppbtInfo.getPointer(0));
            if (rv != 0)
            {
                throw new Exception("C_EX_FreeBuffer failed, error code" + Integer.toHexString(rv));
            }

            System.out.println("Deleting public key");
            pkcs11Wrapper.C_DestroyObject(session, hPubKey);

            System.out.println("Deleting private key");
            pkcs11Wrapper.C_DestroyObject(session, hPrvKey);

            System.out.println("Deleting certificate");
            pkcs11Wrapper.C_DestroyObject(session, hCert);

            System.out.println("Logging out");
            pkcs11Wrapper.C_Logout(session);

            System.out.println("Closing of session");
            pkcs11Wrapper.C_CloseSession(session);

            session = -1;

            System.out.println("Test has been completed successfully");
        }
        catch(Exception e) {
            System.out.printf(e.getMessage() + "\nSome error occurred." );
        }
        try {
            pkcs11Wrapper.C_Logout(session);
        }
        catch(Exception e){}
        try {
            pkcs11Wrapper.C_CloseSession(session);
        }
        catch(Exception e){}
        try{
            pkcs11Wrapper.C_Finalize(null);
        }
        catch(Exception e) {}
    }
}
